/*
 * embuttons.js
 */

var gPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

// Replace BrowserOpenExtensions: Stops a window opening when an XPI is dragged onto a tabbed or sidebar manager
// Also means that the Tools menu items will honour the EMbuttons settings

BrowserOpenExtensions = embuttonsOpenExtensions;

function embuttonsOpenExtensions(aOpenMode)
{

   const OPEN_WINDOW = 0;
   const OPEN_TAB = 1;
   const OPEN_SIDEBAR = 2;
   const EMTYPE = "Extension:Manager";
   const EMFEATURES = "chrome,dialog=no,resizable";
   const EMURL = "chrome://mozapps/content/extensions/extensions.xul?type=" + aOpenMode;
   var anIcon = "chrome://embuttons/skin/images/" + aOpenMode + "-small.png";
   var aSidebar = aOpenMode + 'EMbSidebar'
   var sbElt = document.getElementById(aSidebar);

      var embBundle = document.getElementById("embstrings");
      var extTitle = embBundle.getString("embExtensionsTitle");
      var thmTitle = embBundle.getString("embThemesTitle");
      var panelTitle = (aOpenMode == 'extensions') ? extTitle : thmTitle;

   try {
      var openinPref = (aOpenMode == 'extensions') ? (gPref.getIntPref("extensions.embuttons.extensionsTarget")) : (gPref.getIntPref("extensions.embuttons.themesTarget"));
      var sortPref = (aOpenMode == 'extensions') ? (gPref.getBoolPref("extensions.embuttons.alphasortEM")) : (gPref.getBoolPref("extensions.embuttons.alphasortTM"));
   } catch(e){}

// Check for the matching window - if it's open, focus it and don't try to open another

   var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
   var needToOpen = true;
   var windowType = EMTYPE + "-" + aOpenMode;
   var windows = wm.getEnumerator(windowType);
   while (windows.hasMoreElements()) {
      var theEM = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
      if (theEM.document.documentElement.getAttribute("windowtype") == windowType) {
         theEM.focus();
         needToOpen = false;
         break;
      }
   }

// Check for the matching tab - if it's open, focus it and avoid sorting

   var index = 0;
   var numTabs = getBrowser().mTabContainer.childNodes.length;
   while (index < numTabs) {
      var tempTab = getBrowser().mTabContainer.childNodes[index];

      if (tempTab.getAttribute(aOpenMode) == 'true') {
         if (tempTab.label == panelTitle || tempTab.label == EMURL){
            getBrowser().selectedTab = tempTab;
            needToOpen = false;
            sortPref = false;
            break;
         }else {
            getBrowser().selectedTab = tempTab;
            var url = getWebNavigation().currentURI.spec;
            if (url == EMURL) {
               needToOpen = false;
               sortPref = false;
               break;
            }
            else tempTab.removeAttribute(aOpenMode);
         }
      }
      else index = index+1;
   }

   if (needToOpen) {

// Check for the matching sidebar - if it's open, make sure it gets toggled closed and avoid sorting

      if (sbElt.getAttribute("checked") == "true") {
         openinPref = OPEN_SIDEBAR;
         sortPref = false;
      }

// Sort the extensions or themes

      if (sortPref) SortExtensionsBy('name',(aOpenMode == 'extensions') ? 'extension' : 'theme');

// Open the manager in the prescribed manner, or toggle the sidebar

      switch (openinPref) {
         case OPEN_WINDOW :
            window.openDialog(EMURL, "", EMFEATURES);
            break;
         case OPEN_TAB :
            var newTab = window.getBrowser().addTab(EMURL, null, null);
            newTab.setAttribute(aOpenMode,'true');
            newTab.label = panelTitle;
            newTab.setAttribute("image",anIcon);
            window.getBrowser().selectedTab = newTab;
            break;
         case OPEN_SIDEBAR :
            toggleSidebar(aSidebar);
            break;
      }
   }
}


function SortExtensionsBy(aProp,aType)
{
   var whichRoot = "urn:mozilla:" + aType + ":root"
   var sortTarget = function(aResource,aSortProperty)
   {
      this.resource = aResource;
      this.prop = aSortProperty.toLowerCase();
      this.toString = function(){return this.prop;};
   };

   var RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
               .getService(Components.interfaces.nsIRDFService);
   var Container = Components.classes["@mozilla.org/rdf/container;1"]
               .getService(Components.interfaces.nsIRDFContainer);

   var extensionDS = Components.classes["@mozilla.org/extensions/manager;1"]
               .getService(Components.interfaces.nsIExtensionManager).datasource;
   var root = RDFService.GetResource(whichRoot);
   var arc = RDFService.GetResource("http://www.mozilla.org/2004/em-rdf#"+aProp);
   Container.Init(extensionDS,root);

   var elements = Container.GetElements();
   var sortArray = [];
   try {
      while (elements.hasMoreElements())
      {
         var element=elements.getNext();
         var prop="";
         element.QueryInterface(Components.interfaces.nsIRDFResource);
         var target=extensionDS.GetTarget(element, arc ,true);
         if (target) prop = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
         sortArray.push(new sortTarget(element,prop));
         Container.RemoveElement(element,false);
      }
   } catch(e){}
   sortArray.sort();
   for (var i in sortArray)
      Container.AppendElement(sortArray[i].resource)

   extensionDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
   extensionDS.Flush();
}










